#!/usr/bin/env python
import json
from pathlib import Path
from bs4 import BeautifulSoup
from typing import NamedTuple

def T(fr, en) -> dict:
    return {
        "fr": fr,
        "en": en,
    }


def U(valeur: float | int, unit: str, confidence: float, source:str):
    return {
    "valeur": valeur,
    "unit": unit,
    "confidence": confidence or None,
    "source": source or None,
    }

def get(tag, key):
    try:
        return tag(key)[0].text.strip()
    except IndexError:
        return ""

# ingredients_xml = BeautifulSoup(Path("ingredients.xml").read_text())
components_xml = BeautifulSoup(Path("components.xml").read_text())
# sources_xml = BeautifulSoup(Path("sources.xml").read_text())
# groups_xml = BeautifulSoup(Path("groups.xml").read_text())

def parse_component(c):
    return int(get(c, "const_code")), {
        "display_name": T(get(c, "const_nom_fr"), get(c, "const_nom_eng")),
        "name": get(c, "name"),
        "unit": get(c, "unit"),
    }

def parse_ingredient(a):
    return int(get(a, "alim_code")), {
            "name": T(get(a, "alim_nom_fr"), get(a, "alim_nom_eng")),
            "index_name": get(a, "alim_nom_index_fr"),
            "group": int(get(a, "alim_ssssgrp_code"))
            }

def parse_source(s):
    return int(get(s, "source_code")), get(s, "ref_citation")

# def parse_group(g):
#   return int(get(g, "alim_grp_co

components = { key: component for key, component in [parse_component(c) for c in components_xml("const")] }
# ingredients = { key: compos for key, compos in [parse_ingredient(c) for c in ingredients_xml("alim")]}
# sources = { key: source for key, source in [parse_source(s) for s in sources_xml("sources")]}
# 
Path("components.json").write_text(json.dumps(components, indent=4, ensure_ascii=False))
# Path("ingredients.json").write_text(json.dumps(ingredients, indent=4, ensure_ascii=False))
# Path("sources.json").write_text(json.dumps(sources, indent=4, ensure_ascii=False))

components, ingredients, sources = [json.loads(Path(p+".json").read_text()) for p in ["components", "ingredients", "sources"]]

data = [ {"id": k, **v, "nutrients": {}} for k,v in ingredients.items() ]

def find_by_id(id) -> int|None:
    for i, d in enumerate(data):
        if d["id"] == id:
            return i
    return None


def rows(f, chunksize=1024, sep='|'):
    """
    Read a file where the row separator is '|' lazily.

    Usage:

    >>> with open('big.csv') as f:
    >>>     for r in rows(f):
    >>>         process(r)
    """
    row = ''
    while (chunk := f.read(chunksize)) != '':   # End of file
        while (i := chunk.find(sep)) != -1:     # No separator found
            yield row + chunk[:i]
            chunk = chunk[i+1:]
            row = ''
        row += chunk
    yield row

total = Path("compositions.xml").read_text().count("<COMPO>")
print("\n" * 9)

with open("compositions.xml") as file:
    for i, chunk in enumerate(rows(file, sep="</COMPO>\n   <COMPO>")):
        print(f"\r[{i: 6}/{total: 6}] {i/total*100:.3f}%", end="")
        for c in BeautifulSoup(chunk, features="lxml")("compo"):
            teneur = get(c, "teneur").replace(" ", "").replace(",", ".")
            component = components[get(c, "const_code")]
            unit = component["unit"]
            name = component["name"]
            if not name:
                continue
            less_than = False
            if teneur == "-":
                teneur = None
            elif teneur == "traces":
                teneur = -1
            else:
                if teneur.startswith("<"):
                    less_than = True
                    teneur = teneur.removeprefix("<")
                teneur = float(teneur)
                if unit.startswith("kJ"):
                    teneur *= .2390057
                    unit = unit.replace("kJ", "kcal")
            
            confidence = {"A": 4, "B": 3, "C": 2, "D": 1}.get(get(c, "code_confiance"), 0) / 4
            alim_code = get(c, "alim_code")

            if (idx := find_by_id(alim_code)) and teneur != 0:
                data[idx]["nutrients"][name] = U(teneur, unit, confidence=confidence, source=sources.get(get(c, "source_code")))

        if i % 100_000 == 0 or abs(i - total) < 1000:
            Path("compositions.json").write_text(json.dumps(data, indent=4, ensure_ascii=False))
